-- Filename: 
-- Author: Luke Perkin
-- Date: 2012-05-05

require "libs"
require "lovelibs"
require "NotificationController"
require "EntityController"
require "EntityRender"
require "Hexagon"
require "GridController"
require "SystemController"
require "PortController"
require "ProgramController"
require "lovelibs.loveframes.init"

require "tests"

-- Initialization
function love.load()
    font12 = love.graphics.newFont( 12 )
    font16 = love.graphics.newFont( 16 )
    love.graphics.setFont( font12 )
    hexagon = Hexagon( 100 )
    
    camera1 = camera( vector(16500,11200), 1/32 )
    camera1:setZoomLimits( 1/32, 1 )
    minScale = 1/32
    maxScale = 1
    linear_steps = 5
    maxLinearScale = math.log( 1 / minScale )
    minLinearScale = math.log( 1 / maxScale )
    linear_step = (maxLinearScale - minLinearScale) / linear_steps
    linear_scale = maxLinearScale
    linear_scale_time = 1
    linear_scale_lerp_speed = 1
    target_linear_scale = linear_scale

    love.graphics.setLineStyle( 'smooth' )
end

-- Logic
function love.update(dt)
	EntityController.updateInstances( dt )
	camera1:update( dt )
	loveframes.update( dt )
	tweenToLinearScale( dt )
	tweenToCenter( dt )
end

function tweenToLinearScale( dt )
    if target_linear_scale and linear_scale_time < 1 then
        linear_scale_time = linear_scale_time + (dt * linear_scale_lerp_speed)
        linear_scale = util.lerp( linear_scale, target_linear_scale, linear_scale_time )
        camera1:zoomToCursor( 1/math.exp(linear_scale) )
    end
end

function tweenToCenter( dt )
    if camera1.zoom <= minScale * 1.2 and tweenToCenter_timer then
        tweenToCenter_timer = tweenToCenter_timer + (dt*4)
        local x, y = util.lerpXY( camera1.pos.x, camera1.pos.y, 16500, 11200, dt )
        camera1.pos.x = x
        camera1.pos.y = y
    else
        tweenToCenter_timer = 0
    end
end

-- Rendering
function love.draw()
    --love.graphics.setBackgroundColor(255,255,255)
	love.graphics.setColor(255,255,255)
	camera1:attach()
	EntityRender.renderInstances()
	camera1:detach()
	
	if camera1.zoom < 1/10 then
    	love.graphics.setFont( font16 )
    	love.graphics.setColor(0,0,0)
    	for i, system in ipairs(GridController.instance.systems) do
    	    if #system.programs > 0 then
                local pos = camera1:cameraCoords( system.position )
                love.graphics.print(#system.programs, pos.x, pos.y)
            end
    	end
    	love.graphics.setFont( font12 )
    	love.graphics.setColor(255,255,255)
	end
	
	local s = ("%s systems, %s ports, %s programs. %s zoom, %s fps"):format(
	    #GridController.instance.systems, 
	    #PortController.instances, 
	    #EntityController.getEntitiesByClass( ProgramController ),
	    math.exp(linear_scale),
	    love.timer.getFPS()
	)
	love.graphics.setColor(255,255,255)
	love.graphics.print( s, 10, 5 )
	
	loveframes.draw()
end

function getCurrentCamera()
    return camera1
end

-- Input
function love.keypressed( key, unicode )
	NotificationController.send("on_key_pressed", nil, key, unicode)
	loveframes.keypressed(key,unicode)
end

function love.keyreleased( key, unicode )
	NotificationController.send("on_key_released", nil, key, unicode)
	loveframes.keyreleased(key,unicode)
end

function love.mousepressed( x, y, button )
    NotificationController.send("on_mouse_button_pressed", nil, x, y, button)
    if button == 'r' then
        camera1:startPan()
    end
	if button == 'wu' then
	    -- Zoom in
	    target_linear_scale = target_linear_scale - linear_step
	    linear_scale_time = 0
	    if target_linear_scale < minLinearScale then target_linear_scale = minLinearScale end
	    --camera1:zoomToCursor( 1/math.exp(linear_scale) )
    end
    if button == 'wd' then
        -- Zoom out
        target_linear_scale = target_linear_scale + linear_step
        linear_scale_time = 0
	    if target_linear_scale > maxLinearScale then target_linear_scale = maxLinearScale end
	    --camera1:zoomToCursor( 1/math.exp(linear_scale) )
    end
    loveframes.mousepressed(x,y,button)
end

function love.mousereleased( x, y, button )
    NotificationController.send("on_mouse_button_released", nil, x, y, button)
	if button == 'r' then
	    camera1:endPan()
    end
    loveframes.mousereleased(x,y,button)
end

function love.joystickpressed()
end

function love.joystickreleased()
end